@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2021 Inria                                           *)
+(*  Copyright © 2012-2022 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -24,13 +24,11 @@ open Web_serializable_t
 val create_account : email:string -> user -> account Lwt.t
 val get_account_by_id : int -> account option Lwt.t
 val get_account : user -> account option Lwt.t
-
 val update_account : account -> unit Lwt.t
 val add_update_hook : (account -> unit Lwt.t) -> unit
 
-type capability =
-  | Sudo
+type capability = Sudo
 
 val has_capability : capability -> account -> bool
-
 val check : account -> int list -> bool
+val max_voters : account -> int
