@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2021 Inria                                           *)
+(*  Copyright © 2012-2022 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,27 +19,53 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Belenios_core.Common
-open Belenios_core
-open Serializable_t
+open Belenios
 open Web_serializable_t
 
 module type S = sig
+  val election_home :
+    (module Site_common_sig.ELECTION) ->
+    election_state ->
+    unit ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
 
-  val election_home : (module Site_common_sig.ELECTION_LWT) -> election_state -> unit -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
-  val cast_raw : (module Site_common_sig.ELECTION_LWT) -> unit -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
-  val lost_ballot : (module Site_common_sig.ELECTION_LWT) -> unit -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
-  val cast_confirmed : (module Site_common_sig.ELECTION_LWT) -> result:(user * string * bool * Weight.t * bool, Web_common.error) result -> unit -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
-  val pretty_ballots : (module Site_common_sig.ELECTION_LWT) -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
+  val cast_raw :
+    (module Site_common_sig.ELECTION) ->
+    unit ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
 
-  val booth : unit -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
+  val lost_ballot :
+    (module Site_common_sig.ELECTION) ->
+    unit ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
 
-  val schulze : Question_nh_t.question -> schulze_result -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
+  val cast_confirmed :
+    (module Site_common_sig.ELECTION) ->
+    result:(user * string * bool * Weight.t * bool, Web_common.error) result ->
+    unit ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
 
-  val majority_judgment_select : uuid -> int -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
-  val majority_judgment : Question_nh_t.question -> mj_result -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
+  val pretty_ballots :
+    (module Site_common_sig.ELECTION) ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
+
+  val schulze :
+    Belenios_question.Non_homomorphic.t ->
+    schulze_result ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
+
+  val majority_judgment_select :
+    uuid -> int -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
+
+  val majority_judgment :
+    Belenios_question.Non_homomorphic.t ->
+    mj_result ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
 
   val stv_select : uuid -> int -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
-  val stv : Question_nh_t.question -> stv_result -> [> `Html ] Eliom_content.Html.F.elt Lwt.t
 
+  val stv :
+    Belenios_question.Non_homomorphic.t ->
+    stv_result ->
+    [> `Html ] Eliom_content.Html.F.elt Lwt.t
 end
