@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2022 Inria                                           *)
+(*  Copyright © 2012-2023 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -20,65 +20,43 @@
 (**************************************************************************)
 
 open Lwt.Syntax
-open Belenios_core.Common
-open Belenios_core.Serializable_j
+open Belenios
 open Web_serializable_j
 open Web_common
 
-type 'a t =
-  {
-    of_string : string -> 'a;
-    to_string : 'a -> string;
-    filename : string;
-  }
+type 'a file = {
+  of_string : string -> 'a;
+  to_string : 'a -> string;
+  filename : string;
+}
 
-type 'a list = 'a t
+type 'a abstract = {
+  get : uuid -> 'a option Lwt.t;
+  set : uuid -> 'a -> unit Lwt.t;
+  del : uuid -> unit Lwt.t;
+}
 
-let filename x = x.filename
-
-let raw filename =
-  {
-    of_string = Fun.id;
-    to_string = Fun.id;
-    filename;
-  }
+type 'a t = File of 'a file | Abstract of 'a abstract
 
 let get ~uuid file =
-  let* x = read_file_single_line ~uuid file.filename in
-  let&* x in
-  try Lwt.return_some (file.of_string x)
-  with _ -> Lwt.return_none
-
-let get_default ~default ~uuid file =
-  let* x = get ~uuid file in
-  Lwt.return @@ Option.value ~default x
-
-let get_raw_list ~uuid filename =
-  read_file ~uuid filename.filename
-
-let get_fold_s ~uuid file f accu =
-  let* xs = read_file ~uuid file.filename in
-  let&* xs in
-  let* x = Lwt_list.fold_left_s (fun accu x -> f (file.of_string x) accu) accu xs in
-  Lwt.return_some x
-
-let get_fold_s_default ~uuid file f accu =
-  let* x = read_file ~uuid file.filename in
-  match x with
-  | None -> Lwt.return accu
-  | Some xs -> Lwt_list.fold_left_s (fun accu x -> f (file.of_string x) accu) accu xs
+  match file with
+  | File file -> (
+      let* x = Filesystem.read_file_single_line ~uuid file.filename in
+      let&* x = x in
+      try Lwt.return_some (file.of_string x) with _ -> Lwt.return_none)
+  | Abstract a -> a.get uuid
 
 let set ~uuid file x =
-  write_file ~uuid file.filename [file.to_string x]
-
-let set_list ~uuid file xs =
-  write_file ~uuid file.filename @@ List.map file.to_string xs
+  match file with
+  | File file -> Filesystem.write_file ~uuid file.filename [ file.to_string x ]
+  | Abstract a -> a.set uuid x
 
 let del ~uuid file =
-  let filename = uuid /// file.filename in
-  Lwt.catch
-    (fun () -> Lwt_unix.unlink filename)
-    (fun _ -> Lwt.return_unit)
+  match file with
+  | File file -> Filesystem.cleanup_file (uuid /// file.filename)
+  | Abstract a -> a.del uuid
+
+let make_file x = File x
 
 let draft =
   {
@@ -86,6 +64,21 @@ let draft =
     to_string = string_of_draft_election;
     filename = "draft.json";
   }
+  |> make_file
+
+let draft_public_credentials =
+  let filename = "public_creds.json" in
+  let get uuid = Filesystem.read_whole_file ~uuid filename in
+  let set uuid x = Filesystem.write_file ~uuid filename [ x ] in
+  let del uuid = Filesystem.cleanup_file (uuid /// filename) in
+  Abstract { get; set; del }
+
+let draft_private_credentials =
+  let filename = "private_creds.txt" in
+  let get uuid = Filesystem.read_whole_file ~uuid filename in
+  let set uuid x = Filesystem.write_whole_file ~uuid filename x in
+  let del uuid = Filesystem.cleanup_file (uuid /// filename) in
+  Abstract { get; set; del }
 
 let hide_result =
   {
@@ -93,6 +86,7 @@ let hide_result =
     to_string = string_of_datetime;
     filename = "hide_result";
   }
+  |> make_file
 
 let dates =
   {
@@ -100,6 +94,7 @@ let dates =
     to_string = string_of_election_dates;
     filename = "dates.json";
   }
+  |> make_file
 
 let state =
   {
@@ -107,6 +102,7 @@ let state =
     to_string = string_of_election_state;
     filename = "state.json";
   }
+  |> make_file
 
 let decryption_tokens =
   {
@@ -114,6 +110,7 @@ let decryption_tokens =
     to_string = string_of_decryption_tokens;
     filename = "decryption_tokens.json";
   }
+  |> make_file
 
 let metadata =
   {
@@ -121,15 +118,22 @@ let metadata =
     to_string = string_of_metadata;
     filename = "metadata.json";
   }
+  |> make_file
 
 let private_key =
   {
-    of_string = number_of_string;
-    to_string = string_of_number;
+    of_string = Yojson.Safe.from_string;
+    to_string = Yojson.Safe.to_string;
     filename = "private_key.json";
   }
+  |> make_file
 
-let private_keys = raw "private_keys.jsons"
+let private_keys =
+  let filename = "private_keys.jsons" in
+  let get uuid = Filesystem.read_file ~uuid filename in
+  let set uuid x = Filesystem.write_file ~uuid filename x in
+  let del uuid = Filesystem.cleanup_file (uuid /// filename) in
+  Abstract { get; set; del }
 
 let skipped_shufflers =
   {
@@ -137,6 +141,7 @@ let skipped_shufflers =
     to_string = string_of_skipped_shufflers;
     filename = "skipped_shufflers.json";
   }
+  |> make_file
 
 let shuffle_token =
   {
@@ -144,27 +149,7 @@ let shuffle_token =
     to_string = string_of_shuffle_token;
     filename = "shuffle_token.json";
   }
-
-let extended_records =
-  {
-    of_string = extended_record_of_string;
-    to_string = string_of_extended_record;
-    filename = "extended_records.jsons";
-  }
-
-let records =
-  {
-    of_string = Fun.id;
-    to_string = Fun.id;
-    filename = string_of_election_file ESRecords;
-  }
-
-let credential_mappings =
-  {
-    of_string = credential_mapping_of_string;
-    to_string = string_of_credential_mapping;
-    filename = "credential_mappings.jsons";
-  }
+  |> make_file
 
 let audit_cache =
   {
@@ -172,11 +157,7 @@ let audit_cache =
     to_string = string_of_audit_cache;
     filename = "audit_cache.json";
   }
-
-let voters = raw "voters.txt"
-
-let chain_filename uuid = Uuid.unwrap uuid ^ ".bel"
-let chain uuid = raw (chain_filename uuid)
+  |> make_file
 
 let last_event =
   {
@@ -184,3 +165,12 @@ let last_event =
     to_string = string_of_last_event;
     filename = "last_event.json";
   }
+  |> make_file
+
+let salts =
+  {
+    of_string = salts_of_string;
+    to_string = string_of_salts;
+    filename = "salts.json";
+  }
+  |> make_file
