@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2021 Inria                                           *)
+(*  Copyright © 2012-2022 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -33,6 +33,7 @@ let gdpr_uri = ref ""
 let warning_file = ref None
 let footer_file = ref None
 let admin_home = ref None
+let success_snippet = ref None
 let source_file = ref "belenios.tar.gz"
 let logo = ref None
 let favicon = ref None
@@ -44,3 +45,6 @@ let nh_group = ref ""
 let domain = ref ""
 let deny_revote = ref false
 let deny_newelection = ref false
+let blacklisted_domains = ref Belenios.SSet.empty
+let billing = ref None
+let restricted_mode = ref false
