@@ -6,10 +6,11 @@ import random
 import re
 import warnings
 from urllib.parse import urlencode
+from selenium.webdriver.common.by import By
 from util.selenium_tools import wait_for_element_exists, wait_for_elements_exist, wait_for_element_exists_and_contains_expected_text, wait_for_an_element_with_partial_link_text_exists, verify_element_label, printable_page_source
 from util.election_testing import random_email_addresses_generator, populate_credential_and_password_for_voters_from_sent_emails, populate_random_votes_for_voters, repopulate_vote_confirmations_for_voters_from_sent_emails, wait_a_bit, build_css_selector_to_find_buttons_in_page_content_by_value, find_button_in_page_content_by_value, initialize_browser, election_page_url_to_election_id, verify_election_consistency, create_election_data_snapshot, delete_election_data_snapshot, log_in_as_administrator, log_out, administrator_starts_creation_of_election, administrator_edits_election_questions, administrator_sets_election_voters, administrator_validates_creation_of_election
 from util.execution import console_log
-from util.page_objects import ElectionHomePage, NormalVoteStep1Page, NormalVoteStep2Page, NormalVoteStep3Page, VoterLoginPage, NormalVoteStep5Page, NormalVoteStep6Page, BallotBoxPage, ResponsiveBoothStep1Page, ResponsiveBoothStep2Page, ResponsiveBoothStep3Page
+from util.page_objects import ElectionHomePage, NormalVoteStep1Page, NormalVoteStep2Page, NormalVoteStep3Page, VoterLoginPage, NormalVoteStep6Page, BallotBoxPage, ResponsiveBoothStep1Page, ResponsiveBoothStep2Page, ResponsiveBoothStep3Page
 import settings
 
 
@@ -114,81 +115,14 @@ class BeleniosElectionTestBase(unittest.TestCase):
         generate_on_server_button_element = wait_for_element_exists(browser, generate_on_server_button_css_selector, settings.EXPLICIT_WAIT_TIMEOUT)
         generate_on_server_button_element.click()
 
-        # Wait for emails to be delivered
+        # Wait for credentials to be generated
         time.sleep(10)
 
-        # (Server sends emails to voters.) She checks that server does not show any error that would happen when trying to send these emails (this can happen if sendmail is not configured)
-        confirmation_sentence_expected_text = "Credentials have been generated and mailed!"
+        # She checks that server does not show any error
+        confirmation_sentence_expected_text = "Credentials have been generated!"
         confirmation_sentence_css_selector = "#main p"
         wait_for_element_exists_and_contains_expected_text(browser, confirmation_sentence_css_selector, confirmation_sentence_expected_text, settings.EXPLICIT_WAIT_TIMEOUT)
 
-        # Now we do a sanity check that server has really tried to send emails. For this, we look for email addresses in the temporary file where our fake sendmail executable redirects its inputs to.
-
-        """
-        An email sent by Belenios (using sendmail or using the fake sendmail) to a voter looks like this:
-
-Content-type: text/plain; charset="UTF-8"
-Content-transfer-encoding: quoted-printable
-From: Belenios public server <noreply@example.org>
-To: "820E7G83JBY0F4Z3DY2Y@mailinator.com"
- <820E7G83JBY0F4Z3DY2Y@mailinator.com>
-Subject: Your credential for election My test election for Scenario 1
-MIME-Version: 1.0
-X-Mailer: OcamlNet (ocamlnet.sourceforge.net)
-Date: Wed, 31 Oct 2018 15:22:27 +0100
-
-You are listed as a voter for the election
-
-  My test election for Scenario 1
-
-You will find below your credential.  To cast a vote, you will also
-need a password, sent in a separate email.  Be careful, passwords and
-credentials look similar but play different roles.  You will be asked
-to enter your credential before entering the voting booth.  Login and
-passwords are required once your ballot is ready to be cast.
-
-Credential: yQVDQaKSAQVjdZq
-Page of the election: http://localhost:8001/elections/AFFNDEPnpy21bw/
-
-Note that you are allowed to vote several times.  Only the last vote
-counts.
-
-----------
-
-Vous =C3=AAtes enregistr=C3=A9(e) en tant qu=27=C3=A9lecteur(trice) pour=20=
-l=27=C3=A9lection
-
-  My test election for Scenario 1
-
-Veuillez trouver ci-dessous votre code de vote. Pour soumettre un
-bulletin, vous aurez =C3=A9galement besoin d=27un mot de passe, envoy=C3=A9=
- dans
-un e-mail s=C3=A9par=C3=A9. Soyez attentif(ve), le mot de passe et le cod=
-e de
-vote se ressemblent mais jouent des r=C3=B4les diff=C3=A9rents. Le syst=C3=
-=A8me vous
-demandera votre code de vote d=C3=A8s l=27entr=C3=A9e dans l=27isoloir=20=
-virtuel. Le
-nom d=27utilisateur et le mot de passe sont n=C3=A9cessaires lorsque votr=
-e
-bulletin est pr=C3=AAt =C3=A0 =C3=AAtre soumis.
-
-Code de vote=C2=A0: yQVDQaKSAQVjdZq
-Page de l=27=C3=A9lection=C2=A0: http://localhost:8001/elections/AFFNDEPn=
-py21bw/
-
-Notez que vous pouvez voter plusieurs fois. Seul le dernier vote est
-pris en compte.
-
---=20
-        """
-
-        email_address_to_look_for = self.voters_email_addresses[0]
-        text_to_look_for = 'To: "' + email_address_to_look_for + '"'
-        email_address_found = self.fake_sent_emails_manager.find_in_sent_emails(text_to_look_for)
-        assert email_address_found, "Text '" + email_address_to_look_for + "' not found in fake sendmail log file " + self.fake_sent_emails_manager.log_file_path
-
-
         # She clicks on the "Proceed" link
         proceed_link_css_selector = "#generic_proceed_link"
         proceed_link_element = wait_for_element_exists(browser, proceed_link_css_selector, settings.EXPLICIT_WAIT_TIMEOUT)
@@ -265,7 +199,7 @@ pris en compte.
 
             wait_a_bit()
 
-            # She checks that the page shows a confirmation message similar to "A new password has been mailed to RMR4MY4XV5GUDNOR6XNH@mailinator.com"
+            # She checks that the page shows a confirmation message similar to "A new password has been mailed to RMR4MY4XV5GUDNOR6XNH@example.org"
             confirmation_sentence_expected_text = "A new password has been mailed to"
             confirmation_sentence_css_selector = "#main p"
             wait_for_element_exists_and_contains_expected_text(browser, confirmation_sentence_css_selector, confirmation_sentence_expected_text, settings.EXPLICIT_WAIT_TIMEOUT)
@@ -308,11 +242,7 @@ pris en compte.
         timeout = settings.EXPLICIT_WAIT_TIMEOUT
 
         if direct:
-            booth_url = None
-            if settings.BOOTH_VERSION == settings.BOOTH_VERSIONS.RESPONSIVE_BOOTH:
-                booth_url = settings.SERVER_URL + "/static/frontend/booth/vote.html#" + urlencode({"uuid": self.election_id, "lang": "en"})
-            else:
-                booth_url = settings.SERVER_URL + "/vote.html#" + urlencode({"uuid": self.election_id})
+            booth_url = settings.SERVER_URL + "/static/frontend/booth/vote.html#" + urlencode({"uuid": self.election_id, "lang": "en"})
             browser.get(booth_url)
 
         else:
@@ -330,7 +260,7 @@ pris en compte.
 
         wait_a_bit()
 
-        if settings.BOOTH_VERSION == settings.BOOTH_VERSIONS.RESPONSIVE_BOOTH:
+        if True: # new booth
             # A loading screen appears, then another screen appears. It contains an input for credential and a "Next" button. He types his credential in the input field, and clicks on the "Next" button.
             step_1_page = ResponsiveBoothStep1Page(browser, timeout)
             step_1_page.verify_page()
@@ -394,62 +324,6 @@ pris en compte.
 
             # import time
             # time.sleep(1000)
-        else:
-            # A loading screen appears, then another screen appears. He clicks on the "Here" button. A modal opens (it is an HTML modal created using Window.prompt()), with an input field. He types his credential.
-            step_1_page = NormalVoteStep1Page(browser, timeout)
-            step_1_page.verify_page()
-            step_1_page.click_on_here_button_and_type_voter_credential(voter["credential"])
-
-            wait_a_bit()
-
-            # A new screen appears, which has a title "Step 2/6: Answer to questions", and a content:
-            # "Question 1?"
-            # "Question #1 of 1 — select between 1 and 2 answer(s)"
-            # [ ] "Answer 1"
-            # [ ] "Answer 2"
-            # [Next]
-            # (where "[ ]" is a checkbox, and [Next] is a button)
-            step_2_page = NormalVoteStep2Page(browser, timeout)
-            step_2_page.verify_page()
-
-            # He fills his votes to each answer of the question
-            vote_data = voter["votes"]
-            step_2_page.fill_vote_form(vote_data)
-
-            wait_a_bit()
-
-            # He clicks on the "Next" button
-            step_2_page.click_on_next_button()
-
-            wait_a_bit()
-
-            """
-            A new screen appears, showing:
-
-            Step 3/6: Review and encrypt
-            Question 1?
-            - Answer 1
-
-            Your ballot has been encrypted, but has not been cast yet!
-
-            Your smart ballot tracker is sLRilXoAYcodIrjWrOqPrVXLNlRyCJAqFeeHZ4WCajU
-
-            We invite you to save it in order to check later that it is taken into account.
-
-            [Continue]
-            [Restart]
-            """
-            step_3_page = NormalVoteStep3Page(browser, timeout)
-            step_3_page.verify_page()
-            # He remembers the smart ballot tracker that is displayed.
-            smart_ballot_tracker_value = step_3_page.get_smart_ballot_tracker_value()
-            assert len(smart_ballot_tracker_value) > 5
-            voter["smart_ballot_tracker"] = smart_ballot_tracker_value
-
-            # He clicks on the "Continue" button
-            step_3_page.click_on_continue_button()
-
-            wait_a_bit()
 
         # He arrives on the login page, with a login form (as he has not already logged in during this visit, he does not arrive directly on the step 5 page)
         login_page = VoterLoginPage(browser, timeout)
@@ -460,15 +334,6 @@ pris en compte.
 
         wait_a_bit()
 
-        # He verifies that he is now on step 5 and that page content is correct (page contains 'has been received, but not recorded yet'; page contains a ballot tracker which is the same as the one he noted; page contains voter's username)
-        step_5_page = NormalVoteStep5Page(browser, timeout)
-        step_5_page.verify_page(smart_ballot_tracker_value, voter["username"])
-
-        # He clicks on the "I cast my vote" button
-        step_5_page.click_on_i_cast_my_vote_button()
-
-        wait_a_bit()
-
 
     def some_voters_cast_their_vote(self, voters):
         """
@@ -692,9 +557,9 @@ pris en compte.
             answer_label_css_selector = base_selector + " td:nth-child(1)"
             answer_total_css_selector = base_selector + " td:nth-child(2)"
             answer_expected_label = "Answer " + str(answer_id)
-            answer_element = browser.find_element_by_css_selector(answer_label_css_selector)
+            answer_element = browser.find_element(By.CSS_SELECTOR, answer_label_css_selector)
             verify_element_label(answer_element, answer_expected_label)
-            answer_total_real_value_element = browser.find_element_by_css_selector(answer_total_css_selector)
+            answer_total_real_value_element = browser.find_element(By.CSS_SELECTOR, answer_total_css_selector)
             answer_total_real_value = answer_total_real_value_element.get_attribute('innerText').strip()
             answer_total_expected_value = str(number_of_votes_per_answer['answer' + str(answer_id)])
             assert answer_total_real_value == answer_total_expected_value, "Number of votes for Answer " + str(answer_id) + " displayed on vote result page  (" + answer_total_real_value + ") does not match expected value (" + answer_total_expected_value + "). " + printable_page_source(browser)
