@@ -8,7 +8,10 @@ import shutil
 import subprocess
 import re
 import json
+import email.parser
+import email.policy
 from selenium import webdriver
+from selenium.webdriver.common.by import By
 from selenium.webdriver.common.keys import Keys
 from selenium.common.exceptions import NoSuchElementException
 from util.selenium_tools import wait_for_element_exists, wait_for_element_exists_and_contains_expected_text, wait_for_an_element_with_partial_link_text_exists, verify_element_label, wait_for_element_exists_and_attribute_contains_expected_text, wait_until_page_url_changes
@@ -67,16 +70,17 @@ def populate_credential_and_password_for_voters_from_sent_emails(fake_sent_email
         if len(emails_with_credential) == 0:
             raise Exception("No credential email found for voter " + voter_email_address)
         email_with_credential = emails_with_credential[0]
+        parsed_email = email.parser.Parser(policy=email.policy.default).parsestr(email_with_credential["full_content"][8:]).get_content()
 
         voter_credential = ""
-        match = re.search(r'^Credential: (.*)$', email_with_credential["full_content"], re.MULTILINE)
+        match = re.search(r'^Your credential: (.*)$', parsed_email, re.MULTILINE)
         if match:
             voter_credential = match.group(1)
         else:
             raise Exception("Credential not found in credential email for voter " + voter_email_address)
 
         election_page_url = "" # In this scenario, it looks like all voters receive the same election page URL. Maybe in different scenarios, voters will not all receive the same vote URL (for the same election).
-        match = re.search(r'^Page of the election: (.*)$', email_with_credential["full_content"], re.MULTILINE)
+        match = re.search(r'(http.*/elections/[0-9A-Za-z]+/)$', parsed_email, re.MULTILINE)
         if match:
             election_page_url = match.group(1)
         else:
@@ -223,12 +227,12 @@ def build_css_selector_to_find_buttons_in_page_content_by_value(expected_value):
 
 def find_button_in_page_content_by_value(browser, expected_value):
     css_selector = build_css_selector_to_find_buttons_in_page_content_by_value(expected_value)
-    return browser.find_element_by_css_selector(css_selector)
+    return browser.find_element(By.CSS_SELECTOR, css_selector)
 
 
 def find_buttons_in_page_content_by_value(browser, expected_value):
     css_selector = build_css_selector_to_find_buttons_in_page_content_by_value(expected_value)
-    return browser.find_elements_by_css_selector(css_selector)
+    return browser.find_elements(By.CSS_SELECTOR, css_selector)
 
 
 def initialize_server():
@@ -247,27 +251,23 @@ def initialize_server():
 def initialize_browser(for_scenario_2=False):
     browser = None
 
-    profile = webdriver.FirefoxProfile()
-    profile.set_preference("intl.accept_languages", "en-us")
+    options = webdriver.firefox.options.Options()
+    options.set_preference("intl.accept_languages", "en-us")
 
     if for_scenario_2:
         # Test Scenario 2 requires users to download things from their browser.
         # Define a custom profile for Firefox, to automatically download files that a page asks user to download, without asking. This is because Selenium can't control downloads.
-        profile.set_preference('browser.download.folderList', 2) # Can be set to either 0, 1, or 2. When set to 0, Firefox will save all files downloaded via the browser on the user's desktop. When set to 1, these downloads are stored in the Downloads folder. When set to 2, the location specified for the most recent download is utilized again.
-        profile.set_preference('browser.download.manager.showWhenStarting', False)
-        profile.set_preference('browser.download.dir', settings.BROWSER_DOWNLOAD_FOLDER)
+        options.set_preference('browser.download.folderList', 2) # Can be set to either 0, 1, or 2. When set to 0, Firefox will save all files downloaded via the browser on the user's desktop. When set to 1, these downloads are stored in the Downloads folder. When set to 2, the location specified for the most recent download is utilized again.
+        options.set_preference('browser.download.manager.showWhenStarting', False)
+        options.set_preference('browser.download.dir', settings.BROWSER_DOWNLOAD_FOLDER)
         mime_types_that_should_be_downloaded = ['text/plain', 'application/json']
-        profile.set_preference('browser.helperApps.neverAsk.saveToDisk', ';'.join(mime_types_that_should_be_downloaded))
+        options.set_preference('browser.helperApps.neverAsk.saveToDisk', ';'.join(mime_types_that_should_be_downloaded))
 
     if settings.USE_HEADLESS_BROWSER:
-        from selenium.webdriver.firefox.options import Options
-        options = Options()
         options.add_argument("--headless")
         options.log.level = "trace"
-        browser = webdriver.Firefox(profile, options=options)
-    else:
-        browser = webdriver.Firefox(profile)
-        # browser.maximize_window() # make the browser window use all available screen space. FIXME: When enabled, some clicks are not triggered anymore
+
+    browser = webdriver.Firefox(options=options)
     browser.implicitly_wait(settings.WAIT_TIME_BETWEEN_EACH_STEP) # In seconds
     return browser
 
@@ -328,26 +328,25 @@ def verify_election_consistency(election_id, snapshot_folder=None):
         raise Exception("Error: Verification took longer than " + process_timeout + " seconds. STDOUT was: " + outs + " STDERR was:" + errs)
 
 
-def belenios_tool_generate_credentials(election_id, number_of_voters=None, nh_question=False):
+def belenios_tool_generate_credentials(election_id, voters, nh_question=False):
     """
     Use local CLI belenios-tool to generate a number of credentials corresponding to the number of voters. Example:
     ```
-    ./_run/tool-debug/bin/belenios-tool setup generate-credentials --uuid dmGuNVL1meanZt --group ./files/groups/default.json --count 5
+    ./_run/tool-debug/bin/belenios-tool setup generate-credentials --uuid dmGuNVL1meanZt --group Ed25519 --count 5
     5 private credentials with ids saved to ./1579802689.privcreds
     5 public credentials saved to ./1579802689.pubcreds
     5 hashed public credentials with ids saved to ./1579802689.hashcreds
     ```
     """
 
-    if not number_of_voters:
-        number_of_voters = settings.NUMBER_OF_INVITED_VOTERS
     generated_files_destination_folder = settings.GENERATED_FILES_DESTINATION_FOLDER
+    voters_file = os.path.join(generated_files_destination_folder, "voters.txt")
+    number_of_voters = len(voters.split())
+    with open(voters_file, "w") as f:
+        f.write(voters)
     belenios_tool_path = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, "_run/tool-debug/bin/belenios-tool")
-    if nh_question:
-        crypto_group = "RFC-3526-2048"
-    else:
-        crypto_group = "BELENIOS-2048"
-    command = [belenios_tool_path, "setup", "generate-credentials", "--uuid", election_id, "--group", crypto_group, "--count", str(number_of_voters)]
+    crypto_group = "Ed25519"
+    command = [belenios_tool_path, "setup", "generate-credentials", "--uuid", election_id, "--group", crypto_group, "--file", "voters.txt"]
     running_process = subprocess.Popen(command, cwd=generated_files_destination_folder, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
     process_timeout = 15 * number_of_voters # seconds
     credential_file_id = None
@@ -374,17 +373,10 @@ def belenios_tool_generate_ballots(voters_data, global_credential_file_id, vote_
     i = 0
     for k, v in voters_data.items():
         i += 1
-        # Extract voter private credential from global private credentials file (it corresponds to row `i` in the file) and write it to its own file
+        # Write voter private credential to its own file
         voter_credential_file = os.path.join(generated_files_destination_folder, "voter_row_" + str(i) + "_privcred.txt")
-        command = "tail -n +" + str(i) + " " + global_credential_file_id + ".privcreds | head -n 1 | cut -d' ' -f2 > " + voter_credential_file
-        running_process = subprocess.Popen(command, cwd=generated_files_destination_folder, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True, shell=True)
-        process_timeout = 15 # seconds
-        try:
-            outs, errs = running_process.communicate(timeout=process_timeout)
-        except subprocess.TimeoutExpired:
-            running_process.kill()
-            outs, errs = running_process.communicate()
-            raise Exception("Error: Extraction of voter private credential from global private credentials file took longer than " + str(process_timeout) + " seconds. STDOUT was: " + outs + " STDERR was:" + errs)
+        with open(voter_credential_file, "w") as f:
+            f.write(v["credential"])
 
         # Write array of voter's answers to questions in a file: This is his non-encrypted ballot, written as a JSON array where each element is the answer to the `i`th question. This answer is itself an array of zeros or ones depending on whether voter checked or not the checkbox corresponding to this answer.
         voter_uncrypted_ballot_file = os.path.join(generated_files_destination_folder, "voter_row_" + str(i) + "_uncrypted_ballot.json")
@@ -398,7 +390,7 @@ def belenios_tool_generate_ballots(voters_data, global_credential_file_id, vote_
 
         # Execute belenios-tool to generate a vote ballot for voter
         voter_crypted_ballot_file = "voter_row_" + str(i) + "_crypted_ballot.json"
-        command = [belenios_tool_path, "election", "generate-ballot", "--url", vote_page_url, "--privcred", voter_credential_file, "--ballot", voter_uncrypted_ballot_file, ">", voter_crypted_ballot_file]
+        command = [belenios_tool_path, "election", "generate-ballot", "--url", vote_page_url, "--privcred", voter_credential_file, "--choice", voter_uncrypted_ballot_file, ">", voter_crypted_ballot_file]
         running_process = subprocess.Popen(" ".join(command), cwd=generated_files_destination_folder, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True, shell=True)
         process_timeout = 120 # seconds
         try:
@@ -585,7 +577,7 @@ def administrator_starts_creation_of_election(browser, manual_credential_managem
     wait_a_bit()
 
     election_description_field_css_selector = "#name_and_description_form textarea[name=__co_eliom_description]"
-    election_description_field_element = browser.find_element_by_css_selector(election_description_field_css_selector)
+    election_description_field_element = browser.find_element(By.CSS_SELECTOR, election_description_field_css_selector)
     election_description_field_value = election_description
     election_description_field_element.clear()
     election_description_field_element.send_keys(election_description_field_value)
@@ -594,14 +586,14 @@ def administrator_starts_creation_of_election(browser, manual_credential_managem
 
     # She clicks on the "Save changes" button (the one that is next to the election description field)
     save_changes_button_css_selector = "#name_and_description_form input[type=submit]"
-    save_changes_button_element = browser.find_element_by_css_selector(save_changes_button_css_selector)
+    save_changes_button_element = browser.find_element(By.CSS_SELECTOR, save_changes_button_css_selector)
     save_changes_button_element.click()
 
     wait_a_bit()
 
     # In "Contact" section, she changes the value of "contact" field
     election_contact_field_css_selector = "#form_contact input[name=__co_eliom_contact]"
-    election_contact_field_element = browser.find_element_by_css_selector(election_contact_field_css_selector)
+    election_contact_field_element = browser.find_element(By.CSS_SELECTOR, election_contact_field_css_selector)
     election_contact_field_value = initiator_contact
     election_contact_field_element.clear()
     election_contact_field_element.send_keys(election_contact_field_value)
@@ -610,14 +602,14 @@ def administrator_starts_creation_of_election(browser, manual_credential_managem
 
     # She clicks on the "Save changes" button (the one that is in the "Contact" section)
     contact_section_save_changes_button_css_selector = "#form_contact input[type=submit]"
-    contact_section_save_changes_button_element = browser.find_element_by_css_selector(contact_section_save_changes_button_css_selector)
+    contact_section_save_changes_button_element = browser.find_element(By.CSS_SELECTOR, contact_section_save_changes_button_css_selector)
     contact_section_save_changes_button_element.click()
 
     wait_a_bit()
 
     # In "Public name of the administrator" section, she changes the value of "name" field
     admin_name_field_css_selector = "#form_admin_name input[name=__co_eliom_name]"
-    admin_name_field_element = browser.find_element_by_css_selector(admin_name_field_css_selector)
+    admin_name_field_element = browser.find_element(By.CSS_SELECTOR, admin_name_field_css_selector)
     admin_name_field_value = "Election initiator"
     admin_name_field_element.clear()
     admin_name_field_element.send_keys(admin_name_field_value)
@@ -626,7 +618,7 @@ def administrator_starts_creation_of_election(browser, manual_credential_managem
 
     # She clicks on the "Save changes" button (the one that is in the "Contact" section)
     admin_name_save_changes_button_css_selector = "#form_admin_name input[type=submit]"
-    admin_name_save_changes_button_element = browser.find_element_by_css_selector(admin_name_save_changes_button_css_selector)
+    admin_name_save_changes_button_element = browser.find_element(By.CSS_SELECTOR, admin_name_save_changes_button_css_selector)
     admin_name_save_changes_button_element.click()
 
     wait_a_bit()
@@ -658,39 +650,28 @@ def administrator_edits_election_questions(browser, nh_question=False):
 
     if nh_question:
         # She ticks "Tick the box to activate this mode."
-        browser.find_element_by_css_selector("#hybrid_mode").click()
+        browser.find_element(By.CSS_SELECTOR, "#hybrid_mode").click()
         # She ticks "Alternative"
         nhtally_css_selector = ".nonhomomorphic_tally"
-        nhtally_checkbox_element = browser.find_element_by_css_selector(nhtally_css_selector)
+        nhtally_checkbox_element = browser.find_element(By.CSS_SELECTOR, nhtally_css_selector)
         nhtally_checkbox_element.click()
     else:
         # She sets max to 2
-        max_input = browser.find_element_by_css_selector(".question_max")
+        max_input = browser.find_element(By.CSS_SELECTOR, ".question_max")
         max_input.send_keys(Keys.BACKSPACE)
         max_input.send_keys("2")
 
     # She removes answer 3
     candidate_to_remove = 3
     remove_button_css_selector = ".question_answer_item:nth-child(" + str(candidate_to_remove) + ") .btn_remove"
-    remove_button_element = browser.find_element_by_css_selector(remove_button_css_selector)
+    remove_button_element = browser.find_element(By.CSS_SELECTOR, remove_button_css_selector)
     remove_button_element.click()
 
-    if settings.BOOTH_VERSION == settings.BOOTH_VERSIONS.RESPONSIVE_BOOTH:
-        booth_index = 1
-    else:
-        booth_index = 0
-
-    # In the booth type section, she clicks on the Nth radio button, to select the appropriate booth
-    booth_version_radio_buttons = browser.find_elements_by_css_selector("input[type=radio][name=booth_version_radio]")
-    if not booth_version_radio_buttons or len(booth_version_radio_buttons) != 2:
-        raise Exception("Booth version should be selected among 2 radio buttons")
-    booth_version_radio_buttons[booth_index].click()
-
     wait_a_bit()
 
     # She clicks on the "Save changes" button (this redirects to the "Preparation of election" page)
     save_changes_button_expected_label = "Save changes"
-    button_elements = browser.find_elements_by_css_selector("button")
+    button_elements = browser.find_elements(By.CSS_SELECTOR, "button")
     assert len(button_elements)
     save_changes_button_element = button_elements[-1]
     verify_element_label(save_changes_button_element, save_changes_button_expected_label)
