@@ -49,6 +49,37 @@ def wait_for_an_alert(browser, wait_duration=DEFAULT_WAIT_DURATION):
         raise Exception(f"Could not find expected Alert until timeout of {str(wait_duration)} seconds. " + printable_page_source(browser)) from e
 
 
+class elements_exist_and_are_visible(object):
+    def __init__(self, locator):
+        self.locator = locator
+
+    def __call__(self, driver):
+        elements = driver.find_elements(*self.locator)
+        if not elements:
+            return False
+        visible_elements = list(filter(element_is_visible_filter, elements))
+        if not visible_elements:
+            return False
+        return visible_elements
+
+
+def wait_for_elements_exist_and_are_visible(browser, css_selector, wait_duration=DEFAULT_WAIT_DURATION):
+    """
+    Waits for the presence of elements that match CSS selector `css_selector` and which are currently visible in the page.
+    :param browser: Selenium browser
+    :param css_selector: CSS selector of the expected element
+    :param wait_duration: Maximum duration in seconds that we wait for the presence of this element before raising an exception
+    :return: The list of WebElement once they match expected conditions
+    """
+    try:
+        ignored_exceptions = (NoSuchElementException, StaleElementReferenceException,)
+        custom_wait = WebDriverWait(browser, wait_duration, ignored_exceptions=ignored_exceptions)
+        elements = custom_wait.until(elements_exist_and_are_visible((By.CSS_SELECTOR, css_selector)))
+        return elements
+    except Exception as e:
+        raise Exception(f"Could not find expected visible DOM elements matching '{css_selector}' until timeout of {str(wait_duration)} seconds. " + printable_page_source(browser)) from e
+
+
 class an_element_exists_and_is_visible_and_attribute_contains_expected_text(object):
     def __init__(self, locator, attribute_name, expected_content):
         self.locator = locator
@@ -140,7 +171,7 @@ class an_element_with_partial_link_text_exists(object):
         self.partial_link_text = partial_link_text
 
     def __call__(self, driver):
-        element = driver.find_element_by_partial_link_text(self.partial_link_text)
+        element = driver.find_element(By.PARTIAL_LINK_TEXT, self.partial_link_text)
         if not element:
             return False
         return element
@@ -151,7 +182,7 @@ class an_element_with_link_text_exists(object):
         self.link_text = link_text
 
     def __call__(self, driver):
-        element = driver.find_element_by_link_text(self.link_text)
+        element = driver.find_element(By.LINK_TEXT, self.link_text)
         if not element:
             return False
         return element
@@ -184,7 +215,7 @@ class element_exists_and_attribute_contains_expected_text(object):
         if not element:
             return False
         element_content = self.transform_attribute_content(element.get_attribute(self.attribute))
-        if self.expected_text in element_content:
+        if element_content and self.expected_text in element_content:
             return element
         else:
             return False
@@ -207,7 +238,10 @@ class element_exists_and_contains_expected_text(element_exists_and_attribute_con
         super().__init__(locator, 'innerText', expected_text)
 
     def transform_attribute_content(self, content):
-        return content.strip()
+        if content:
+            return content.strip()
+        else:
+            return content
 
 
 class element_exists_and_does_not_contain_expected_text(object):
@@ -359,6 +393,15 @@ def wait_for_elements_exist(browser, css_selector, wait_duration=DEFAULT_WAIT_DU
         raise Exception("Could not find expected DOM elements '" + css_selector + "' until timeout of " + str(wait_duration) + " seconds." + printable_page_source(browser)) from e
 
 
+def wait_until_page_url_changes(browser, old_url, wait_duration=DEFAULT_WAIT_DURATION):
+    try:
+        return WebDriverWait(browser, wait_duration).until(
+            lambda driver: old_url != driver.current_url
+        )
+    except Exception as e:
+        raise Exception("Could not detect a change in page URL until timeout of " + str(wait_duration) + " seconds." + printable_page_source(browser)) from e
+
+
 def set_element_attribute(browser, element_dom_id, attribute_key, attribute_value):
     browser.execute_script("let el = document.getElementById('" + element_dom_id + "'); el.setAttribute('" + attribute_key + "','" + attribute_value + "');")
 
